@@ -6,9 +6,7 @@ use Doctrine\Common\Annotations\Annotation;
 use Doctrine\Common\Annotations\Annotation\NamedArgumentConstructor;
 use Doctrine\Common\Annotations\Annotation\Target;
 use Doctrine\Common\Annotations\AnnotationException;
-use Doctrine\Common\Annotations\AnnotationRegistry;
 use Doctrine\Common\Annotations\DocParser;
-use Doctrine\Common\Annotations\NamedArgumentConstructorAnnotation;
 use Doctrine\Tests\Common\Annotations\Fixtures\AnnotationTargetAll;
 use Doctrine\Tests\Common\Annotations\Fixtures\AnnotationWithConstants;
 use Doctrine\Tests\Common\Annotations\Fixtures\ClassWithConstants;
@@ -130,7 +128,7 @@ class DocParserTest extends TestCase
         self::assertEquals('value2', $annot->value[1]['key2']);
 
         // Complete docblock
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * Some nifty class.
  *
@@ -174,7 +172,7 @@ DOCBLOCK;
         $parser = new DocParser();
         $parser->setIgnoreNotImportedAnnotations(true);
 
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * Some nifty class.
  *
@@ -200,14 +198,14 @@ DOCBLOCK;
     {
         $parser = $this->createTestParser();
 
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * Some nifty method.
  *
  * @since 2.0
  * @Doctrine\Tests\Common\Annotations\Name(foo="bar")
- * @param string \$foo This is foo.
- * @param mixed \$bar This is bar.
+ * @param string $foo This is foo.
+ * @param mixed $bar This is bar.
  * @return string Foo and bar.
  * @This is irrelevant
  * @Marker
@@ -229,7 +227,7 @@ DOCBLOCK;
     {
         $parser = $this->createTestParser();
 
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @SomeAnnotationClassNameWithoutConstructor("Some data")
  */
@@ -245,7 +243,7 @@ DOCBLOCK;
         self::assertNotNull($annot->data);
         self::assertEquals($annot->data, 'Some data');
 
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @SomeAnnotationClassNameWithoutConstructor(name="Some Name", data = "Some data")
  */
@@ -261,7 +259,7 @@ DOCBLOCK;
         self::assertEquals($annot->name, 'Some Name');
         self::assertEquals($annot->data, 'Some data');
 
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @SomeAnnotationClassNameWithoutConstructor(data = "Some data")
  */
@@ -274,7 +272,7 @@ DOCBLOCK;
         self::assertEquals($annot->data, 'Some data');
         self::assertNull($annot->name);
 
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @SomeAnnotationClassNameWithoutConstructor(name = "Some name")
  */
@@ -287,7 +285,7 @@ DOCBLOCK;
         self::assertEquals($annot->name, 'Some name');
         self::assertNull($annot->data);
 
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @SomeAnnotationClassNameWithoutConstructor("Some data")
  */
@@ -300,7 +298,7 @@ DOCBLOCK;
         self::assertEquals($annot->data, 'Some data');
         self::assertNull($annot->name);
 
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @SomeAnnotationClassNameWithoutConstructor("Some data",name = "Some name")
  */
@@ -313,7 +311,7 @@ DOCBLOCK;
         self::assertEquals($annot->name, 'Some name');
         self::assertEquals($annot->data, 'Some data');
 
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @SomeAnnotationWithConstructorWithoutParams(name = "Some name")
  */
@@ -326,7 +324,7 @@ DOCBLOCK;
         self::assertEquals($annot->name, 'Some name');
         self::assertEquals($annot->data, 'Some data');
 
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @SomeAnnotationClassNameWithoutConstructorAndProperties()
  */
@@ -1058,7 +1056,7 @@ ANNOTATION
     public function testWithoutConstructorWhenIsNotDefaultValue(): void
     {
         $parser   = $this->createTestParser();
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @SomeAnnotationClassNameWithoutConstructorAndProperties("Foo")
  */
@@ -1076,7 +1074,7 @@ DOCBLOCK;
     public function testWithoutConstructorWhenHasNoProperties(): void
     {
         $parser   = $this->createTestParser();
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @SomeAnnotationClassNameWithoutConstructorAndProperties(value = "Foo")
  */
@@ -1095,7 +1093,7 @@ DOCBLOCK;
     {
         $parser   = $this->createTestParser();
         $context  = 'class SomeClassName';
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @Doctrine\Tests\Common\Annotations\Fixtures\AnnotationWithTargetSyntaxError()
  */
@@ -1114,7 +1112,7 @@ DOCBLOCK;
     {
         $parser   = $this->createTestParser();
         $context  = 'class SomeClassName';
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @AnnotationWithInvalidTargetDeclaration()
  */
@@ -1142,7 +1140,7 @@ DOCBLOCK;
     {
         $parser   = $this->createTestParser();
         $context  = 'class SomeClassName';
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @AnnotationWithTargetEmpty()
  */
@@ -1171,7 +1169,7 @@ DOCBLOCK;
     {
         $parser = $this->createTestParser();
 
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @Name
  *
@@ -1183,7 +1181,7 @@ DOCBLOCK;
 
         self::assertInstanceOf(Name::class, $result[0]);
 
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @Name
  * @Marker
@@ -1315,11 +1313,6 @@ DOCBLOCK;
 
         $parser = new DocParser();
 
-        AnnotationRegistry::registerAutoloadNamespace(
-            'Doctrine\Tests\Common\Annotations\Fixtures\Annotation',
-            __DIR__ . '/../../../../'
-        );
-
         $parser->setImports([
             'autoload' => Fixtures\Annotation\Autoload::class,
         ]);
@@ -1359,7 +1352,7 @@ DOCBLOCK;
      */
     public function testSyntaxErrorWithUnknownCharacters(): void
     {
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @test at.
  */
@@ -1384,7 +1377,7 @@ DOCBLOCK;
      */
     public function testIgnorePHPDocThrowTag(): void
     {
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @throws \RuntimeException
  */
@@ -1454,7 +1447,7 @@ DOCBLOCK;
 
     public function testSetValuesException(): void
     {
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  * @SomeAnnotationClassNameWithoutConstructor(invalidaProperty = "Some val")
  */
@@ -1494,7 +1487,7 @@ Available properties: data, name'
 
     public function testTabPrefixIsAllowed(): void
     {
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
 /**
  *	@Name
  */
@@ -1578,7 +1571,7 @@ DOCBLOCK;
             $this->markTestSkipped('This test requires mbstring function overloading is turned on');
         }
 
-        $docblock = <<<DOCBLOCK
+        $docblock = <<<'DOCBLOCK'
         /**
          * Мультибайтовый текст ломал парсер при оверлоадинге строковых функций
          * @Doctrine\Tests\Common\Annotations\Name
@@ -1608,42 +1601,6 @@ DOCBLOCK;
         self::assertInstanceOf(SomeAnnotationClassNameWithoutConstructorAndProperties::class, $result[0]);
     }
 
-    public function testNamedArgumentsConstructorInterface(): void
-    {
-        $result = $this
-            ->createTestParser()
-            ->parse('/** @NamedAnnotation(foo="baz", bar=2222) */');
-
-        self::assertCount(1, $result);
-        self::assertInstanceOf(NamedAnnotation::class, $result[0]);
-        self::assertSame('baz', $result[0]->getFoo());
-        self::assertSame(2222, $result[0]->getBar());
-    }
-
-    public function testNamedReorderedArgumentsConstructorInterface(): void
-    {
-        $result = $this
-            ->createTestParser()
-            ->parse('/** @NamedAnnotation(bar=2222, foo="baz") */');
-
-        self::assertCount(1, $result);
-        self::assertInstanceOf(NamedAnnotation::class, $result[0]);
-        self::assertSame('baz', $result[0]->getFoo());
-        self::assertSame(2222, $result[0]->getBar());
-    }
-
-    public function testNamedArgumentsConstructorInterfaceWithDefaultValue(): void
-    {
-        $result = $this
-            ->createTestParser()
-            ->parse('/** @NamedAnnotation(foo="baz") */');
-
-        self::assertCount(1, $result);
-        self::assertInstanceOf(NamedAnnotation::class, $result[0]);
-        self::assertSame('baz', $result[0]->getFoo());
-        self::assertSame(1234, $result[0]->getBar());
-    }
-
     public function testNamedArgumentsConstructorAnnotation(): void
     {
         $result = $this
@@ -1692,6 +1649,18 @@ DOCBLOCK;
         self::assertSame(1234, $result[0]->getBar());
     }
 
+    public function testNamedArgumentsConstructorAnnotationWithExtraArguments(): void
+    {
+        $docParser = $this->createTestParser();
+
+        $this->expectException(AnnotationException::class);
+        $this->expectExceptionMessageMatches(
+            '/does not have a property named "invalid"\s.*\sAvailable named arguments: foo, bar/'
+        );
+
+        $docParser->parse('/** @AnotherNamedAnnotation(foo="baz", invalid="uh oh") */');
+    }
+
     public function testNamedArgumentsConstructorAnnotationWithDefaultPropertyAsArray(): void
     {
         $result = $this
@@ -1744,6 +1713,115 @@ DOCBLOCK;
         }
     }
 
+    public function testAnnotationWithConstructorWithVariadicParamAndExtraNamedArguments(): void
+    {
+        $parser   = $this->createTestParser();
+        $docblock = <<<'DOCBLOCK'
+/**
+ * @SomeAnnotationWithConstructorWithVariadicParam(name = "Some data", foo = "Foo", bar = "Bar")
+ */
+DOCBLOCK;
+
+        $this->expectException(AnnotationException::class);
+        $this->expectExceptionMessageMatches(
+            '/does not have a property named "foo"\s.*\sAvailable named arguments: name/'
+        );
+
+        $parser->parse($docblock);
+    }
+
+    public function testAnnotationWithConstructorWithVariadicParamAndExtraNamedArgumentsShuffled(): void
+    {
+        $parser   = $this->createTestParser();
+        $docblock = <<<'DOCBLOCK'
+/**
+ * @SomeAnnotationWithConstructorWithVariadicParam(foo = "Foo", name = "Some data", bar = "Bar")
+ */
+DOCBLOCK;
+
+        $this->expectException(AnnotationException::class);
+        $this->expectExceptionMessageMatches(
+            '/does not have a property named "foo"\s.*\sAvailable named arguments: name/'
+        );
+
+        $parser->parse($docblock);
+    }
+
+    public function testAnnotationWithConstructorWithVariadicParamAndCombinedNamedAndPositionalArguments(): void
+    {
+        $parser   = $this->createTestParser();
+        $docblock = <<<'DOCBLOCK'
+/**
+ * @SomeAnnotationWithConstructorWithVariadicParam("Some data", "Foo", bar = "Bar")
+ */
+DOCBLOCK;
+
+        $this->expectException(AnnotationException::class);
+        $this->expectExceptionMessageMatches(
+            '/does not have a property named "bar"\s.*\sAvailable named arguments: name/'
+        );
+
+        $parser->parse($docblock);
+    }
+
+    public function testAnnotationWithConstructorWithVariadicParamPassOneNamedArgument(): void
+    {
+        $parser   = $this->createTestParser();
+        $docblock = <<<'DOCBLOCK'
+/**
+ * @SomeAnnotationWithConstructorWithVariadicParam(name = "Some data", data = "Foo")
+ */
+DOCBLOCK;
+
+        $this->expectException(AnnotationException::class);
+        $this->expectExceptionMessageMatches(
+            '/does not have a property named "data"\s.*\sAvailable named arguments: name/'
+        );
+
+        $parser->parse($docblock);
+    }
+
+    public function testAnnotationWithConstructorWithVariadicParamPassPositionalArguments(): void
+    {
+        $parser   = $this->createTestParser();
+        $docblock = <<<'DOCBLOCK'
+/**
+ * @SomeAnnotationWithConstructorWithVariadicParam("Some data", "Foo", "Bar")
+ */
+DOCBLOCK;
+
+        $result = $parser->parse($docblock);
+        self::assertCount(1, $result);
+        $annot = $result[0];
+
+        self::assertInstanceOf(SomeAnnotationWithConstructorWithVariadicParam::class, $annot);
+
+        self::assertSame('Some data', $annot->name);
+        // Positional extra arguments will be ignored
+        self::assertSame([], $annot->data);
+    }
+
+    public function testAnnotationWithConstructorWithVariadicParamNoArgs(): void
+    {
+        $parser = $this->createTestParser();
+
+        // Without variadic arguments
+        $docblock = <<<'DOCBLOCK'
+/**
+ * @SomeAnnotationWithConstructorWithVariadicParam("Some data")
+ */
+DOCBLOCK;
+
+        $result = $parser->parse($docblock);
+        self::assertCount(1, $result);
+        $annot = $result[0];
+
+        self::assertInstanceOf(SomeAnnotationWithConstructorWithVariadicParam::class, $annot);
+
+        self::assertSame('Some data', $annot->name);
+        self::assertSame([], $annot->data);
+    }
+
     /**
      * Override for BC with PHPUnit <8
      */
@@ -1757,31 +1835,6 @@ DOCBLOCK;
     }
 }
 
-/** @Annotation */
-class NamedAnnotation implements NamedArgumentConstructorAnnotation
-{
-    /** @var string */
-    private $foo;
-    /** @var int */
-    private $bar;
-
-    public function __construct(string $foo, int $bar = 1234)
-    {
-        $this->foo = $foo;
-        $this->bar = $bar;
-    }
-
-    public function getFoo(): string
-    {
-        return $this->foo;
-    }
-
-    public function getBar(): int
-    {
-        return $this->bar;
-    }
-}
-
 /**
  * @Annotation
  * @NamedArgumentConstructor
@@ -1850,6 +1903,25 @@ class NamedAnnotationWithArray
     }
 }
 
+/**
+ * @Annotation
+ * @NamedArgumentConstructor
+ */
+class SomeAnnotationWithConstructorWithVariadicParam
+{
+    public function __construct(string $name, string ...$data)
+    {
+        $this->name = $name;
+        $this->data = $data;
+    }
+
+    /** @var string[] */
+    public $data;
+
+    /** @var string */
+    public $name;
+}
+
 /** @Annotation */
 class SettingsAnnotation
 {
